import { EditorView } from "@codemirror/view"
import { Extension } from "@codemirror/state"
import { LanguageSupport } from "@codemirror/language"
import { default as React_2 } from "react"
import { SQLConfig } from "@codemirror/lang-sql"

declare interface CommonProps {
  value?: string
  defaultValue?: string
  onChange?: (value: unknown) => void
  /**
   * @deprecated
   */
  extraCompletions?:
    | string[]
    | ((completion: string) => string[] | Promise<string[]>)
  theme?: "dark" | "light"
  editable?: boolean
  extensions?: (Extension | LanguageSupport)[]
  diagnostics?: ExtraDiagnostic
  [key: string]: unknown
}

declare type DiagnosticItem = {
  error: string
  errorCode: number
  lineOffsets: LineOffset[]
}

declare type DiagnosticResult = {
  checkItems: DiagnosticItem[]
  rawSQL: string
}

declare interface ExtraDiagnostic {
  checkNo?: string
  checkResult: DiagnosticResult
  funcOSSList?: unknown[]
  parsedTableResult?: {
    funcs?: string | null
    inCSVs?: string | null
    inHbases?: unknown[]
    inRDBs?: unknown[]
    kafkaItem?: unknown[]
    sinkKafkas?: unknown[]
  }
  withFunc?: boolean
}

export declare type IEditor = EditorView

/**
 * format 方法接受的参数
 */
export declare type IFormatOptions = {
  parser(text: string): string | Promise<string>
}

declare interface JavascriptProps extends CommonProps {
  language: "javascript"
  langOptions?: {
    jsx?: boolean
    typescript?: boolean
  }
}

declare interface JsonProps extends CommonProps {
  language: "json"
  langOptions?: never
}

declare type LineOffset = {
  EndLine: number
  EndOffset: number
  StartLine: number
  StartOffset: number
}

export declare const ReactCodemirror: React_2.ForwardRefExoticComponent<
  (ReactCodemirrorProps | StaticCodemirrorProps) &
    React_2.RefAttributes<ReactCodemirrorRefValues>
>

declare type ReactCodemirrorProps =
  | JavascriptProps
  | JsonProps
  | SqlProps

export declare type ReactCodemirrorRefValues = {
  readonly editor?: EditorView
  format(configs: IFormatOptions): void
}

declare interface SqlProps extends CommonProps {
  language: "sql"
  langOptions?: SQLConfig
}

declare interface StaticCodemirrorProps extends CommonProps {
  editable: false
  onChange: never
  defaultValue: never
  language: ReactCodemirrorProps["language"]
  langOptions: ReactCodemirrorProps["langOptions"]
}

export {}
